@@ -2,7 +2,7 @@
  * dpkg-deb - construction and deconstruction of *.deb archives
  * dpkg-deb.h - external definitions for this program
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -23,7 +23,8 @@
 #define DPKG_DEB_H
 
 typedef void dofunction(const char *const *argv);
-dofunction do_build, do_contents, do_control;
+dofunction do_build NONRETURNING; 
+dofunction do_contents, do_control, do_showinfo;
 dofunction do_info, do_field, do_extract, do_vextract, do_fsystarfile;
 
 extern int debugflag, nocheckflag, oldformatflag;
@@ -34,11 +35,19 @@ void extracthalf(const char *debar, const char *directory,
                  const char *taroption, int admininfo);
 
 extern const char *compression;
+extern const char* showformat;
+extern enum compression_type compress_type;
 
 #define DEBMAGIC     "!<arch>\ndebian-binary   "
 #define ADMINMEMBER		"control.tar.gz  "
 #define ADMINMEMBER_COMPAT	"control.tar.gz/ "
-#define DATAMEMBER		"data.tar.gz     "
-#define DATAMEMBER_COMPAT	"data.tar.gz/    "
+#define DATAMEMBER_GZ		"data.tar.gz     "
+#define DATAMEMBER_COMPAT_GZ	"data.tar.gz/    "
+#define DATAMEMBER_BZ2   	"data.tar.bz2    "
+#define DATAMEMBER_COMPAT_BZ2  	"data.tar.bz2/   "
+#define DATAMEMBER_LZMA		"data.tar.lzma   "
+#define DATAMEMBER_COMPAT_LZMA	"data.tar.lzma/  "
+#define DATAMEMBER_CAT   	"data.tar        "
+#define DATAMEMBER_COMPAT_CAT  	"data.tar/       "
 
 #endif /* DPKG_DEB_H */
