@@ -1,8 +1,8 @@
 /*
- * dselect - Debian GNU/Linux package maintenance user interface
+ * dselect - Debian package maintenance user interface
  * methkeys.cc - method list keybindings
  *
- * Copyright (C) 1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -18,13 +18,15 @@
  * License along with this; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+extern "C" {
+#include <config.h>
+}
 
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 
 extern "C" {
-#include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
 }
@@ -110,6 +112,7 @@ const keybindings::orgbinding methodlist_korgbindings[]= {
   { '\r',           "select-and-quit"  },
   { 'x',            "abort"            },
   { 'X',            "abort"            },
+  { 'Q',            "abort"            },
   
   {  -1,             0                 }
 };
