@@ -18,13 +18,15 @@
  * License along with this; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+extern "C" {
+#include <config.h>
+}
 
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
 
 extern "C" {
-#include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
 }
@@ -70,6 +72,7 @@ const char
 			    N_("recommends"),
 			    N_("depends on"),
 			    N_("pre-depends on"),
+			    N_("breaks with"),
 			    N_("conflicts with"),
 			    N_("provides"),
 			    N_("replaces"),
@@ -184,6 +187,8 @@ void packagelist::setwidths() {
   }
     
   total_width= TOTAL_LIST_WIDTH;
+  if (total_width < COLS)
+    total_width= COLS;
   description_width= total_width - description_column;
 }
 
