@@ -119,10 +119,10 @@ int checksubprocerr(int status, const char *description, int flags) {
     n= WTERMSIG(status); if (!n || ((flags & PROCPIPE) && n==SIGPIPE)) return 0;
     if (flags & PROCWARN)
       ohshit(_("dpkg: warning - %s killed by signal (%s)%s\n"),
-           description, strsignal(n), WCOREDUMP(status) ? ", core dumped" : "");
+	     description, strsignal(n), WCOREDUMP(status) ? _(", core dumped") : "");
     else
       ohshit(_("subprocess %s killed by signal (%s)%s"),
-           description, strsignal(n), WCOREDUMP(status) ? ", core dumped" : "");
+	     description, strsignal(n), WCOREDUMP(status) ? _(", core dumped") : "");
   } else {
     ohshit(_("subprocess %s failed with wait status code %d"),description,status);
   }
@@ -176,7 +176,7 @@ off_t buffer_write(buffer_data_t data, void *buf, off_t length, const char *desc
 	  unsigned char *hash = *ctx->hash = malloc(33);
 	  MD5Final(digest, &ctx->ctx);
 	  for (i = 0; i < 16; ++i) {
-	    sprintf(hash, "%02x", *p++);
+	    sprintf((char *)hash, "%02x", *p++);
 	    hash += 2;
 	  }
 	  *hash = 0;
@@ -189,7 +189,7 @@ off_t buffer_write(buffer_data_t data, void *buf, off_t length, const char *desc
   switch(data->type) {
     case BUFFER_WRITE_BUF:
       memcpy(data->data.ptr, buf, length);
-      (char*)data->data.ptr += length;
+      data->data.ptr += length;
       break;
     case BUFFER_WRITE_VBUF:
       varbufaddbuf((struct varbuf *)data->data.ptr, buf, length);
