@@ -3,47 +3,54 @@
 $version= '1.0.11'; # This line modified by Makefile
 $admindir= "/var/lib/dpkg"; # This line modified by Makefile
 $dpkglibdir= "../utils"; # This line modified by Makefile
+($0) = $0 =~ m:.*/(.+):;
 
-$enoent=`$dpkglibdir/enoent` || die "Cannot get ENOENT value from $dpkglibdir/enoent: $!";
+push (@INC, $dpkglibdir);
+require 'dpkg-gettext.pl';
+textdomain("dpkg");
+
+$enoent=`$dpkglibdir/enoent` || die sprintf(_g("Cannot get ENOENT value from %s: %s"), "$dpkglibdir/enoent", $!);
 sub ENOENT { $enoent; }
 
-sub showversion {
-    print("Debian dpkg-divert $version.\n") || &quit("failed to write version: $!");
+sub version {
+    printf _g("Debian %s version %s.\n"), $0, $version;
+
+    printf _g("
+Copyright (C) 1995 Ian Jackson.
+Copyright (C) 2000,2001 Wichert Akkerman.");
+
+    printf _g("
+This is free software; see the GNU General Public Licence version 2 or
+later for copying conditions. There is NO warranty.
+");
 }
 
 sub usage {
-    &showversion;
-    print STDERR <<EOF
-Copyright (C) 1995 Ian Jackson.
-Copyright (C) 2000,2001 Wichert Akkerman.
+    printf(_g(
+"Usage: %s [<option> ...] <command>
 
-This is free software; see the GNU General Public Licence version 2 or later
-for copying conditions. There is NO warranty.
+Commands:
+  [--add] <file>           add a diversion.
+  --remove <file>          remove the diversion.
+  --list [<glob-pattern>]  show file diversions.
+  --truename <file>        return the diverted file.
 
-Usage:
-
- dpkg-divert [options] [--add] <file>               - add a diversion
- dpkg-divert [options] --remove <file>              - remove the diversion
- dpkg-divert [options] --list [<glob-pattern>]      - show file diversions
- dpkg-divert [options] --truename <file>            - return the diverted file
-
-Options: 
-    --package <package>        name of the package whose copy of <file>
-                               will not be diverted.
-    --local                    all packages' versions are diverted.
-    --divert <divert-to>       the name used by other packages' versions.
-    --rename                   actually move the file aside (or back).
-    --quiet                    quiet operation, minimal output
-    --test                     don't do anything, just demonstrate
-    --help                     print this help screen and exit
-    --version                  output version and exit
-    --admindir <directory>     set the directory with the diversions file
+Options:
+  --package <package>      name of the package whose copy of <file> will not
+                             be diverted.
+  --local                  all packages' versions are diverted.
+  --divert <divert-to>     the name used by other packages' versions.
+  --rename                 actually move the file aside (or back).
+  --admindir <directory>   set the directory with the diversions file.
+  --test                   don't do anything, just demonstrate.
+  --quiet                  quiet operation, minimal output.
+  --help                   show this help message.
+  --version                show the version.
 
 When adding, default is --local and --divert <original>.distrib.
 When removing, --package or --local and --divert must match if specified.
 Package preinst/postrm scripts should always specify --package and --divert.
-EOF
-        || &quit("failed to write usage: $!");
+"), $0);
 }
 
 $testmode= 0;
@@ -54,7 +61,7 @@ $|=1;
 
 sub checkmanymodes {
     return unless $mode;
-    &badusage("two modes specified: $_ and --$mode");
+    &badusage(sprintf(_g("two modes specified: %s and --%s"), $_, $mode));
 }
 
 while (@ARGV) {
@@ -65,7 +72,7 @@ while (@ARGV) {
     } elsif (m/^--help$/) {
         &usage; exit(0);
     } elsif (m/^--version$/) {
-        &showversion; exit(0);
+        &version; exit(0);
     } elsif (m/^--test$/) {
         $testmode= 1;
     } elsif (m/^--rename$/) {
@@ -87,71 +94,71 @@ while (@ARGV) {
         &checkmanymodes;
         $mode= 'truename';
     } elsif (m/^--divert$/) {
-        @ARGV || &badusage("--divert needs a divert-to argument");
+        @ARGV || &badusage(sprintf(_g("--%s needs a divert-to argument"), "divert"));
         $divertto= shift(@ARGV);
-        $divertto =~ m/\n/ && &badusage("divert-to may not contain newlines");
+        $divertto =~ m/\n/ && &badusage(_g("divert-to may not contain newlines"));
     } elsif (m/^--package$/) {
-        @ARGV || &badusage("--package needs a package argument");
+        @ARGV || &badusage(sprintf(_g("--%s needs a <package> argument"), "package"));
         $package= shift(@ARGV);
-        $package =~ m/\n/ && &badusage("package may not contain newlines");
+        $package =~ m/\n/ && &badusage(_g("package may not contain newlines"));
     } elsif (m/^--admindir$/) {
-        @ARGV || &badusage("--admindir needs a directory argument");
+        @ARGV || &badusage(sprintf(_g("--%s needs a <directory> argument"), "admindir"));
         $admindir= shift(@ARGV);
     } else {
-        &badusage("unknown option \`$_'");
+        &badusage(sprintf(_g("unknown option \`%s'"), $_));
     }
 }
 
 $mode='add' unless $mode;
 
-open(O,"$admindir/diversions") || &quit("cannot open diversions: $!");
+open(O,"$admindir/diversions") || &quit(sprintf(_g("cannot open diversions: %s"), $!));
 while(<O>) {
     s/\n$//; push(@contest,$_);
-    $_=<O>; s/\n$// || &badfmt("missing altname");
+    $_=<O>; s/\n$// || &badfmt(_g("missing altname"));
     push(@altname,$_);
-    $_=<O>; s/\n$// || &badfmt("missing package");
+    $_=<O>; s/\n$// || &badfmt(_g("missing package"));
     push(@package,$_);
 }
 close(O);
 
 if ($mode eq 'add') {
-    @ARGV == 1 || &badusage("--add needs a single argument");
+    @ARGV == 1 || &badusage(sprintf(_g("--%s needs a single argument"), "add"));
     $file= $ARGV[0];
-    $file =~ m#^/# || &badusage("filename \"$file\" is not absolute");
-    $file =~ m/\n/ && &badusage("file may not contain newlines");
-	-d $file && &badusage("Cannot divert directories");
+    $file =~ m#^/# || &badusage(sprintf(_g("filename \"%s\" is not absolute"), $file));
+    $file =~ m/\n/ && &badusage(_g("file may not contain newlines"));
+	-d $file && &badusage(_g("Cannot divert directories"));
     $divertto= "$file.distrib" unless defined($divertto);
-    $divertto =~ m#^/# || &badusage("filename \"$divertto\" is not absolute");
+    $divertto =~ m#^/# || &badusage(sprintf(_g("filename \"%s\" is not absolute"), $divertto));
     $package= ':' unless defined($package);
     for ($i=0; $i<=$#contest; $i++) {
         if ($contest[$i] eq $file || $altname[$i] eq $file ||
             $contest[$i] eq $divertto || $altname[$i] eq $divertto) {
             if ($contest[$i] eq $file && $altname[$i] eq $divertto &&
                 $package[$i] eq $package) {
-                print "Leaving \`",&infon($i),"'\n" if $verbose > 0;
+                printf(_g("Leaving \`%s'")."\n", &infon($i)) if $verbose > 0;
                 exit(0);
             }
-            &quit("\`".&infoa."' clashes with \`".&infon($i)."'");
+            &quit(sprintf(_g("\`%s' clashes with \`%s'"), &infoa, &infon($i)));
         }
     }
     push(@contest,$file);
     push(@altname,$divertto);
     push(@package,$package);
-    print "Adding \`",&infon($#contest),"'\n" if $verbose > 0;
+    printf(_g("Adding \`%s'")."\n", &infon($#contest)) if $verbose > 0;
     &checkrename($file,$divertto);
     &save;
     &dorename($file,$divertto);
     exit(0);
 } elsif ($mode eq 'remove') {
-    @ARGV == 1 || &badusage("--remove needs a single argument");
+    @ARGV == 1 || &badusage(sprintf(_g("--%s needs a single argument"), "remove"));
     $file= $ARGV[0];
     for ($i=0; $i<=$#contest; $i++) {
         next unless $file eq $contest[$i];
-        &quit("mismatch on divert-to\n  when removing \`".&infoa."'\n  found \`".
-              &infon($i)."'") if defined($divertto) && $altname[$i] ne $divertto;
-        &quit("mismatch on package\n  when removing \`".&infoa."'\n  found \`".
-              &infon($i)."'") if defined($package) && $package[$i] ne $package;
-        print "Removing \`",&infon($i),"'\n" if $verbose > 0;
+        &quit(sprintf(_g("mismatch on divert-to\n  when removing \`%s'\n  found \`%s'"), &infoa, &infon($i)))
+              if defined($divertto) && $altname[$i] ne $divertto;
+        &quit(sprintf(_g("mismatch on package\n  when removing \`%s'\n  found \`%s'"), &infoa, &infon($i)))
+              if defined($package) && $package[$i] ne $package;
+        printf(_g("Removing \`%s'")."\n", &infon($i)) if $verbose > 0;
         $orgfile= $contest[$i];
         $orgdivertto= $altname[$i];
         @contest= (($i > 0 ? @contest[0..$i-1] : ()),
@@ -166,7 +173,7 @@ if ($mode eq 'add') {
         &save;
         exit(0);
     }
-    print "No diversion \`",&infoa,"', none removed\n" if $verbose > 0;
+    printf(_g("No diversion \`%s', none removed")."\n", &infoa) if $verbose > 0;
     exit(0);
 } elsif ($mode eq 'list') {
     @ilist= @ARGV ? @ARGV : ('*');
@@ -185,7 +192,7 @@ if ($mode eq 'add') {
     }
     exit(0);
 } elsif ($mode eq 'truename') {
-    @ARGV == 1 || &badusage("--truename needs a single argument");
+    @ARGV == 1 || &badusage(sprintf(_g("--%s needs a single argument"), "truename"));
     $file= $ARGV[0];
     for ($i=0; $i<=$#contest; $i++) {
 	next unless $file eq $contest[$i];
@@ -195,7 +202,7 @@ if ($mode eq 'add') {
     print $file, "\n";
     exit(0);
 } else {
-    &quit("internal error - bad mode \`$mode'");
+    &quit(sprintf(_g("internal error - bad mode \`%s'"), $mode));
 }
 
 sub infol {
@@ -209,9 +216,9 @@ sub checkrename {
     return unless $dorename;
     ($rsrc,$rdest) = @_;
     (@ssrc= lstat($rsrc)) || $! == &ENOENT ||
-        &quit("cannot stat old name \`$rsrc': $!");
+        &quit(sprintf(_g("cannot stat old name \`%s': %s"), $rsrc, $!));
     (@sdest= lstat($rdest)) || $! == &ENOENT ||
-        &quit("cannot stat new name \`$rdest': $!");
+        &quit(sprintf(_g("cannot stat new name \`%s': %s"), $rdest, $!));
     # Unfortunately we have to check for write access in both
     # places, just having +w is not enough, since people do
     # mount things RO, and we need to fail before we start
@@ -226,13 +233,13 @@ sub checkrename {
 	} elsif ($! == ENOENT) {
 		$dorename = !$dorename;
 	} else {
-		&quit("error checking \`$file': $!");
+		&quit(sprintf(_g("error checking \`%s': %s"), $file, $!));
 	}
     }
     if (@ssrc && @sdest &&
         !($ssrc[0] == $sdest[0] && $ssrc[1] == $sdest[1])) {
-        &quit("rename involves overwriting \`$rdest' with\n".
-              "  different file \`$rsrc', not allowed");
+        &quit(sprintf(_g("rename involves overwriting \`%s' with\n".
+              "  different file \`%s', not allowed"), $rdest, $rsrc));
     }
 }
 
@@ -241,33 +248,44 @@ sub dorename {
     return if $testmode;
     if (@ssrc) {
         if (@sdest) {
-            unlink($rsrc) || &quit("rename: remove duplicate old link \`$rsrc': $!");
+            unlink($rsrc) || &quit(sprintf(_g("rename: remove duplicate old link \`%s': %s"), $rsrc, $!));
         } else {
-            rename($rsrc,$rdest) || &quit("rename: rename \`$rsrc' to \`$rdest': $!");
+            rename($rsrc,$rdest) || &quit(sprintf(_g("rename: rename \`%s' to \`%s': %s"), $rsrc, $rdest, $!));
         }
     }
 }            
     
 sub save {
     return if $testmode;
-    open(N,"> $admindir/diversions-new") || &quit("create diversions-new: $!");
+    open(N,"> $admindir/diversions-new") || &quit(sprintf(_g("create diversions-new: %s"), $!));
     chmod 0644, "$admindir/diversions-new";
     for ($i=0; $i<=$#contest; $i++) {
         print(N "$contest[$i]\n$altname[$i]\n$package[$i]\n")
-            || &quit("write diversions-new: $!");
+            || &quit(sprintf(_g("write diversions-new: %s"), $!));
     }
-    close(N) || &quit("close diversions-new: $!");
+    close(N) || &quit(sprintf(_g("close diversions-new: %s"), $!));
     unlink("$admindir/diversions-old") ||
-        $! == &ENOENT || &quit("remove old diversions-old: $!");
+        $! == &ENOENT || &quit(sprintf(_g("remove old diversions-old: %s"), $!));
     link("$admindir/diversions","$admindir/diversions-old") ||
-        $! == &ENOENT || &quit("create new diversions-old: $!");
+        $! == &ENOENT || &quit(sprintf(_g("create new diversions-old: %s"), $!));
     rename("$admindir/diversions-new","$admindir/diversions")
-        || &quit("install new diversions: $!");
+        || &quit(sprintf(_g("install new diversions: %s"), $!));
 }
 
 sub infoa { &infol($file,$divertto,$package); }
 sub infon { &infol($contest[$i],$altname[$i],$package[$i]); }
 
-sub quit { print STDERR "dpkg-divert: @_\n"; exit(2); }
-sub badusage { print STDERR "dpkg-divert: @_\n\n"; print("You need --help.\n"); exit(2); }
-sub badfmt { &quit("internal error: $admindir/diversions corrupt: $_[0]"); }
+sub quit
+{
+    printf STDERR "%s: %s\n", $0, "@_";
+    exit(2);
+}
+
+sub badusage
+{
+    printf STDERR "%s: %s\n\n", $0, "@_";
+    &usage;
+    exit(2);
+}
+
+sub badfmt { &quit(sprintf(_g("internal error: %s corrupt: %s"), "$admindir/diversions", $_[0])); }
