@@ -15,34 +15,44 @@ use POSIX qw(:errno_h);
 push(@INC,$dpkglibdir);
 require 'controllib.pl';
 
-sub usageversion {
-    print STDERR
-"Debian dpkg-gencontrol $version. 
-Copyright 1996 Ian Jackson.
-Copyright 2000,2002 Wichert Akkerman.
-This is free software; see the GNU General Public Licence version 2 or later
-for copying conditions.  There is NO warranty.
+require 'dpkg-gettext.pl';
+textdomain("dpkg-dev");
 
-Usage: dpkg-gencontrol [options ...]
+sub version {
+    printf _g("Debian %s version %s.\n"), $progname, $version;
 
-Options:  -p<package>            print control file for package
-          -c<controlfile>        get control info from this file
-          -l<changelogfile>      get per-version info from this file
-          -F<changelogformat>    force change log format
-          -v<forceversion>       set version of binary package
-          -f<fileslistfile>      write files here instead of debian/files
-          -P<packagebuilddir>    temporary build dir instead of debian/tmp
-	  -n<filename>           assume the package filename will be <filename>
-          -O                     write to stdout, not .../DEBIAN/control
-          -is                    include section field
-          -ip                    include priority field
-          -isp|-ips              include both section and priority
-          -D<field>=<value>      override or add a field and value
-          -U<field>              remove a field
-          -V<name>=<value>       set a substitution variable
-          -T<varlistfile>        read variables here, not debian/substvars
-          -h                     print this message
-";
+    printf _g("
+Copyright (C) 1996 Ian Jackson.
+Copyright (C) 2000,2002 Wichert Akkerman.");
+
+    printf _g("
+This is free software; see the GNU General Public Licence version 2 or
+later for copying conditions. There is NO warranty.
+");
+}
+
+sub usage {
+    printf _g(
+"Usage: %s [<option> ...]
+
+Options:
+  -p<package>              print control file for package.
+  -c<controlfile>          get control info from this file.
+  -l<changelogfile>        get per-version info from this file.
+  -F<changelogformat>      force change log format.
+  -v<forceversion>         set version of binary package.
+  -f<fileslistfile>        write files here instead of debian/files.
+  -P<packagebuilddir>      temporary build dir instead of debian/tmp.
+  -n<filename>             assume the package filename will be <filename>.
+  -O                       write to stdout, not .../DEBIAN/control.
+  -is, -ip, -isp, -ips     deprecated, ignored for compatibility.
+  -D<field>=<value>        override or add a field and value.
+  -U<field>                remove a field.
+  -V<name>=<value>         set a substitution variable.
+  -T<varlistfile>          read variables here, not debian/substvars.
+  -h, --help               show this help message.
+      --version            show the version.
+"), $progname;
 }
 
 $i=100;grep($fieldimps{$_}=$i--,
@@ -57,7 +67,7 @@ while (@ARGV) {
     if (m/^-p([-+0-9a-z.]+)$/) {
         $oppackage= $1;
     } elsif (m/^-p(.*)/) {
-        &error("Illegal package name \`$1'");
+        &error(sprintf(_g("Illegal package name \`%s'"), $1));
     } elsif (m/^-c/) {
         $controlfile= $';
     } elsif (m/^-l/) {
@@ -70,13 +80,8 @@ while (@ARGV) {
         $forceversion= $1;
     } elsif (m/^-O$/) {
         $stdout= 1;
-    } elsif (m/^-is$/) {
-        $spinclude{'Section'}=1;
-    } elsif (m/^-ip$/) {
-        $spinclude{'Priority'}=1;
-    } elsif (m/^-isp$/ || m/^-ips$/) {
-        $spinclude{'Section'}=1;
-        $spinclude{'Priority'}=1;
+    } elsif (m/^-i[sp][sp]?$/) {
+	# ignored for backwards compatibility
     } elsif (m/^-F([0-9a-z]+)$/) {
         $changelogformat=$1;
     } elsif (m/^-D([^\=:]+)[=:]/) {
@@ -89,10 +94,12 @@ while (@ARGV) {
         $varlistfile= $';
     } elsif (m/^-n/) {
         $forcefilename= $';
-    } elsif (m/^-h$/) {
-        &usageversion; exit(0);
+    } elsif (m/^-(h|-help)$/) {
+        &usage; exit(0);
+    } elsif (m/^--version$/) {
+        &version; exit(0);
     } else {
-        &usageerr("unknown option \`$_'");
+        &usageerr(sprintf(_g("unknown option \`%s'"), $_));
     }
 }
 
@@ -102,12 +109,12 @@ while (@ARGV) {
 &parsecontrolfile;
             
 if (length($oppackage)) {
-    defined($p2i{"C $oppackage"}) || &error("package $oppackage not in control info");
+    defined($p2i{"C $oppackage"}) || &error(sprintf(_g("package %s not in control info"), $oppackage));
     $myindex= $p2i{"C $oppackage"};
 } else {
     @packages= grep(m/^C /,keys %p2i);
     @packages==1 ||
-        &error("must specify package since control info has many (@packages)");
+        &error(sprintf(_g("must specify package since control info has many (%s)"), "@packages"));
     $myindex=1;
 }
 
@@ -124,7 +131,7 @@ for $_ (keys %fi) {
         elsif (s/^X[CS]*B[CS]*-//i) { $f{$_}= $v; }
 	elsif (m/^X[CS]+-|^(Standards-Version|Uploaders)$|^Build-(Depends|Conflicts)(-Indep)?$/i) { }
 	elsif (m/^Section$|^Priority$/) { $spdefault{$_}= $v; }
-        else { $_ = "C $_"; &unknown('general section of control info file'); }
+        else { $_ = "C $_"; &unknown(_g('general section of control info file')); }
     } elsif (s/^C$myindex //) {
 #print STDERR "P key >$_< value >$v<\n";
         if (m/^(Package|Description|Essential|Optional)$/) {
@@ -133,21 +140,29 @@ for $_ (keys %fi) {
         } elsif (m/^Section$|^Priority$/) {
             $spvalue{$_}= $v;
         } elsif (m/^Architecture$/) {
-            if ($v eq 'all') {
+            if (debian_arch_eq('all', $v)) {
                 $f{$_}= $v;
-            } elsif ($v eq 'any') {
+            } elsif (debian_arch_is($arch, $v)) {
                 $f{$_}= $arch;
             } else {
                 @archlist= split(/\s+/,$v);
-                grep($arch eq $_, @archlist) ||
-                    &error("current build architecture $arch does not".
-                           " appear in package's list (@archlist)");
+		my @invalid_archs = grep m/[^\w-]/, @archlist;
+		&warn(sprintf(ngettext(
+		                  "`%s' is not a legal architecture string.",
+		                  "`%s' are not legal architecture strings.",
+		                  scalar(@invalid_archs)),
+		              join("' `", @invalid_archs)))
+		    if @invalid_archs >= 1;
+                grep(debian_arch_is($arch, $_), @archlist) ||
+                    &error(sprintf(_g("current build architecture %s does not".
+                                      " appear in package's list (%s)"),
+                                   $arch, "@archlist"));
                 $f{$_}= $arch;
             }
         } elsif (s/^X[CS]*B[CS]*-//i) {
             $f{$_}= $v;
         } elsif (!m/^X[CS]+-/i) {
-            $_ = "C$myindex $_"; &unknown("package's section of control info file");
+            $_ = "C$myindex $_"; &unknown(_g("package's section of control info file"));
         }
     } elsif (m/^C\d+ /) {
 #print STDERR "X key >$_< value not shown<\n";
@@ -162,19 +177,17 @@ for $_ (keys %fi) {
         } elsif (s/^X[CS]*B[CS]*-//i) {
             $f{$_}= $v;
         } elsif (!m/^X[CS]+-/i) {
-            $_ = "L $_"; &unknown("parsed version of changelog");
+            $_ = "L $_"; &unknown(_g("parsed version of changelog"));
         }
     } elsif (m/o:/) {
     } else {
-        &internerr("value from nowhere, with key >$_< and value >$v<");
+        &internerr(sprintf(_g("value from nowhere, with key >%s< and value >%s<"), $_, $v));
     }
 }
 
 $f{'Version'}= $forceversion if length($forceversion);
-$version= $f{'Version'};
-$origversion= $version; $origversion =~ s/-[^-]+$//;
-$substvar{"dpkg:UpstreamVersion"}=$origversion;
-$substvar{"dpkg:Version"}=$version;
+
+&init_substvars;
 
 for $_ (keys %fi) {
     $v= $fi{$_};
@@ -182,7 +195,9 @@ for $_ (keys %fi) {
     } elsif (s/^C$myindex //) {
         if (m/^(Package|Description|Essential|Optional)$/) {
         } elsif (exists($pkg_dep_fields{$_})) {
-            $f{$_}= showdep(parsedep(substvars($v)), 0);
+           my $dep = parsedep(substvars($v), 1, 1);
+           &error(sprintf(_g("error occurred while parsing %s"), $_)) unless defined $dep;
+            $f{$_}= showdep($dep, 0);
         } elsif (m/^Section$|^Priority$/) {
         } elsif (m/^Architecture$/) {
         } elsif (s/^X[CS]*B[CS]*-//i) {
@@ -198,32 +213,33 @@ for $_ (keys %fi) {
 
 for $f (qw(Section Priority)) {
     $spvalue{$f}= $spdefault{$f} unless length($spvalue{$f});
-    $f{$f}= $spvalue{$f} if $spinclude{$f} && length($spvalue{$f});
+    $f{$f}= $spvalue{$f} if length($spvalue{$f});
 }
 
 for $f (qw(Package Version)) {
-    defined($f{$f}) || &error("missing information for output field $f");
+    defined($f{$f}) || &error(sprintf(_g("missing information for output field %s"), $f));
 }
 for $f (qw(Maintainer Description Architecture)) {
-    defined($f{$f}) || &warn("missing information for output field $f");
+    defined($f{$f}) || &warn(sprintf(_g("missing information for output field %s"), $f));
 }
 $oppackage= $f{'Package'};
 
-$verdiff= $f{'Version'} ne $sourceversion;
+$verdiff = $f{'Version'} ne $substvar{'source:Version'} or
+           $f{'Version'} ne $sourceversion;
 if ($oppackage ne $sourcepackage || $verdiff) {
     $f{'Source'}= $sourcepackage;
-    $f{'Source'}.= " ($sourceversion)" if $verdiff;
+    $f{'Source'}.= " ($substvar{'source:Version'})" if $verdiff;
 }
 
 if (!defined($substvar{'Installed-Size'})) {
-    defined($c= open(DU,"-|")) || &syserr("fork for du");
+    defined($c= open(DU,"-|")) || &syserr(_g("fork for du"));
     if (!$c) {
-        chdir("$packagebuilddir") || &syserr("chdir for du to \`$packagebuilddir'");
-        exec("du","-k","-s","."); &syserr("exec du");
+        chdir("$packagebuilddir") || &syserr(sprintf(_g("chdir for du to \`%s'"), $packagebuilddir));
+        exec("du","-k","-s",".") or &syserr(_g("exec du"));
     }
     $duo=''; while (<DU>) { $duo.=$_; }
-    close(DU); $? && &subprocerr("du in \`$packagebuilddir'");
-    $duo =~ m/^(\d+)\s+\.$/ || &failure("du gave unexpected output \`$duo'");
+    close(DU); $? && &subprocerr(sprintf(_g("du in \`%s'"), $packagebuilddir));
+    $duo =~ m/^(\d+)\s+\.$/ || &failure(sprintf(_g("du gave unexpected output \`%s'"), $duo));
     $substvar{'Installed-Size'}= $1;
 }
 if (defined($substvar{'Extra-Size'})) {
@@ -233,22 +249,27 @@ if (length($substvar{'Installed-Size'})) {
     $f{'Installed-Size'}= $substvar{'Installed-Size'};
 }
 
+for $f (keys %override) { $f{&capit($f)}= $override{$f}; }
+for $f (keys %remove) { delete $f{&capit($f)}; }
+
 $fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
-open(Y,"> $fileslistfile.new") || &syserr("open new files list file");
+open(Y,"> $fileslistfile.new") || &syserr(_g("open new files list file"));
 binmode(Y);
 chown(@fowner, "$fileslistfile.new") 
-		|| &syserr("chown new files list file");
+		|| &syserr(_g("chown new files list file"));
 if (open(X,"< $fileslistfile")) {
     binmode(X);
     while (<X>) {
         chomp;
-        next if m/^([-+0-9a-z.]+)_[^_]+_(\w+)\.deb /
-                && ($1 eq $oppackage) && ($2 eq $arch || $2 eq 'all');
-        print(Y "$_\n") || &syserr("copy old entry to new files list file");
+        next if m/^([-+0-9a-z.]+)_[^_]+_([\w-]+)\.deb /
+                && ($1 eq $oppackage)
+                && (debian_arch_eq($2, $f{'Architecture'})
+		    || debian_arch_eq($2, 'all'));
+        print(Y "$_\n") || &syserr(_g("copy old entry to new files list file"));
     }
-    close(X) || &syserr("close old files list file");
+    close(X) || &syserr(_g("close old files list file"));
 } elsif ($! != ENOENT) {
-    &syserr("read old files list file");
+    &syserr(_g("read old files list file"));
 }
 $sversion=$f{'Version'};
 $sversion =~ s/^\d+://;
@@ -256,23 +277,20 @@ $forcefilename=sprintf("%s_%s_%s.deb", $oppackage,$sversion,$f{'Architecture'})
 	   unless ($forcefilename);
 print(Y &substvars(sprintf("%s %s %s\n", $forcefilename, 
                            &spfileslistvalue('Section'), &spfileslistvalue('Priority'))))
-    || &syserr("write new entry to new files list file");
-close(Y) || &syserr("close new files list file");
-rename("$fileslistfile.new",$fileslistfile) || &syserr("install new files list file");
-
-for $f (keys %override) { $f{&capit($f)}= $override{$f}; }
-for $f (keys %remove) { delete $f{&capit($f)}; }
+    || &syserr(_g("write new entry to new files list file"));
+close(Y) || &syserr(_g("close new files list file"));
+rename("$fileslistfile.new",$fileslistfile) || &syserr(_g("install new files list file"));
 
 if (!$stdout) {
     $cf= "$packagebuilddir/DEBIAN/control";
     $cf= "./$cf" if $cf =~ m/^\s/;
     open(STDOUT,"> $cf.new") ||
-        &syserr("cannot open new output control file \`$cf.new'");
+        &syserr(sprintf(_g("cannot open new output control file \`%s'"), "$cf.new"));
     binmode(STDOUT);
 }
 &outputclose(1);
 if (!$stdout) {
-    rename("$cf.new","$cf") || &syserr("cannot install output control file \`$cf'");
+    rename("$cf.new","$cf") || &syserr(sprintf(_g("cannot install output control file \`%s'"), $cf));
 }
 
 sub spfileslistvalue {
