@@ -2,34 +2,56 @@
 
 use Text::Wrap;
 
-# fixme: --dirfile option
+my $dpkglibdir = "."; # This line modified by Makefile
+push (@INC, $dpkglibdir);
+require 'dpkg-gettext.pl';
+textdomain("dpkg");
+
+($0) = $0 =~ m:.*/(.+):;
+
 # fixme: sort entries
 # fixme: send to FSF ?
 
 $version= '0.93.42.2'; # This line modified by Makefile
 sub version {
-        $file = $_[0];
-        print $file <<END;
-Debian install-info $version.  Copyright (C) 1994,1995
-Ian Jackson.  This is free software; see the GNU General Public Licence
-version 2 or later for copying conditions.  There is NO warranty.
-END
+    printf _g("Debian %s version %s.\n"), $0, $version;
+
+    printf _g("
+Copyright (C) 1994,1995 Ian Jackson.");
+
+    printf _g("
+This is free software; see the GNU General Public Licence version 2 or
+later for copying conditions. There is NO warranty.
+");
 }
 
 sub usage {
     $file = $_[0];
-    print $file <<END;
-usage: install-info [--version] [--help] [--debug] [--maxwidth=nnn]
-             [--section regexp title] [--infodir=xxx] [--align=nnn]
-             [--calign=nnn] [--quiet] [--menuentry=xxx] [--info-dir=xxx]
-             [--keep-old] [--description=xxx] [--test]
-             [--remove | --remove-exactly ] [--dir-file]
-             [--]
-             filename
-END
+    printf $file _g(
+"Usage: %s [<options> ...] [--] <filename>
+
+Options:
+  --section <regexp> <title>
+                           put the new entry in the <regex> matched section
+                           or create a new one with <title> if non-existent.
+  --menuentry=<text>       set the menu entry.
+  --description=<text>     set the description to be used in the menu entry.
+  --info-file=<path>       specify info file to install in the directory.
+  --dir-file=<path>        specify file name of info directory file.
+  --infodir=<directory>    same as '--dir-file=<directory>/dir'.
+  --info-dir=<directory>   likewise.
+  --keep-old               do not replace entries nor remove empty ones.
+  --remove                 remove the entry specified by <filename> basename.
+  --remove-exactly         remove the exact <filename> entry.
+  --test                   enables test mode (no actions taken).
+  --debug                  enables debug mode (show more information).
+  --quiet                  do not show output messages.
+  --help                   show this help message.
+  --version                show the version.
+"), $0;
 }
 
-$infodir='/usr/share/info';
+$dirfile = '/usr/share/info/dir';
 $maxwidth=79;
 $Text::Wrap::columns=$maxwidth;
 $backup='/var/backups/infodir.bak';
@@ -68,49 +90,51 @@ while ($ARGV[0] =~ m/^--/) {
         $remove_exactly=1;
     } elsif ($_ eq '--help') {
         &usage(STDOUT); exit 0;
+    } elsif ($_ eq '--version') {
+        &version; exit 0;
     } elsif ($_ eq '--debug') {
-        open(DEBUG,">&STDERR") || die "Could not open stderr for output! $!\n";
+	open(DEBUG,">&STDERR")
+	    || &quit(sprintf(_g("could not open stderr for output! %s"), $!));
 	$debug=1;
     } elsif ($_ eq '--section') {
         if (@ARGV < 2) {
-            print STDERR "$name: --section needs two more args\n";
+	    printf STDERR _g("%s: --section needs two more args")."\n", $name;
             &usage(STDERR); exit 1;
         }
         $sectionre= shift(@ARGV);
         $sectiontitle= shift(@ARGV);
     } elsif (m/^--(c?align|maxwidth)=([0-9]+)$/) {
-	warn( "$name: $1 deprecated(ignored)\n" );
-    } elsif (m/^--infodir=/) {
-        $infodir=$';
+	warn(sprintf(_g("%s: option --%s is deprecated (ignored)"), $name, $1)."\n");
+    } elsif (m/^--info-?dir=/) {
+	$dirfile = $' . '/dir';
     } elsif (m/^--info-file=/) {
-        $filename=$';
+	$filename = $';
     } elsif (m/^--menuentry=/) {
-        $menuentry=$';
-    } elsif (m/^--info-dir=/) {
-        $infodir=$';
+	$menuentry = $';
     } elsif (m/^--description=/) {
-        $description=$';
+	$description = $';
     } elsif (m/^--dir-file=/) { # for compatibility with GNU install-info
-	$infodir=$';
+	$dirfile = $';
     } else {
-        print STDERR "$name: unknown option \`$_'\n"; &usage(STDERR); exit 1;
+        printf STDERR _g("%s: unknown option \`%s'")."\n", $name, $_;
+        &usage(STDERR); exit 1;
     }
 }
 
-if (!@ARGV) { &version(STDERR); print STDERR "\n"; &usage(STDERR); exit 1; }
+if (!@ARGV) { &usage(STDERR); exit 1; }
 
 if ( !$filename ) {
 	$filename= shift(@ARGV);
 	$name = "$name($filename)";
 }
-if (@ARGV) { print STDERR "$name: too many arguments\n"; &usage(STDERR); exit 1; }
+if (@ARGV) { printf STDERR _g("%s: too many arguments")."\n", $name; &usage(STDERR); exit 1; }
 
 if ($remove) {
-    print STDERR "$name: --section ignored with --remove\n" if length($sectiontitle);
-    print STDERR "$name: --description ignored with --remove\n" if length($description);
+    printf(STDERR _g("%s: --section ignored with --remove")."\n", $name) if length($sectiontitle);
+    printf(STDERR _g("%s: --description ignored with --remove")."\n", $name) if length($description);
 }
 
-print STDERR "$name: test mode - dir file will not be updated\n"
+printf(STDERR _g("%s: test mode - dir file will not be updated")."\n", $name)
     if $nowrite && !$quiet;
 
 umask(umask(0777) & ~0444);
@@ -124,7 +148,9 @@ $filename =~ m|[^/]+$|; $basename= $&; $basename =~ s/(\.info)?(\.gz)?$//;
 # The location of the info files from the dir entry, i.e. (emacs-20/emacs).
 my $fileinentry;
 
-&dprint("infodir='$infodir'  filename='$filename'  maxwidth='$maxwidth'\nmenuentry='$menuentry'  basename='$basename'\ndescription='$description'  remove=$remove");
+&dprint("dirfile='$dirfile' filename='$filename' maxwidth='$maxwidth'");
+&dprint("menuentry='$menuentry' basename='$basename'");
+&dprint("description='$description' remove=$remove");
 
 if (!$remove) {
 
@@ -136,7 +162,7 @@ if (!$remove) {
 
     if (!length($description)) {
         
-        open(IF,"$filename") || die "$name: read $filename: $!\n";
+        open(IF,"$filename") || &quit(sprintf(_g("unable to read %s: %s"), $filename, $!));
         $asread='';
         while(<IF>) {
 	    m/^START-INFO-DIR-ENTRY$/ && last;
@@ -158,13 +184,11 @@ if (!$remove) {
             &dprint("multiline '$asread'");
         } elsif ($asread =~ m/^\*\s*([^:]+):(\s*\(([^\)]+)\)\.|:)\s*/) {
             $menuentry= $1;
-            $description= $';
+            $description = $';
             $fileinentry = $3;
             &dprint("infile menuentry '$menuentry' description '$description'");
         } elsif (length($asread)) {
-            print STDERR <<END;
-$name: warning, ignoring confusing INFO-DIR-ENTRY in file.
-END
+            printf STDERR _g("%s: warning, ignoring confusing INFO-DIR-ENTRY in file.")."\n", $name;
         }
     }
 
@@ -173,18 +197,18 @@ END
         $infoentry =~ m/\n/;
         print "$`\n" unless $quiet;
         $infoentry =~ m/^\*\s*([^:]+):\s*\(([^\)]+)\)/ || 
-            die "$name: Invalid info entry\n"; # internal error
+            &quit(_g("invalid info entry")); # internal error
         $sortby= $1;
         $fileinentry= $2;
 
     } else {
-        
+
         if (!length($description)) {
-            open(IF,"$filename") || die "$name: read $filename: $!\n";
+            open(IF,"$filename") || &quit(_g("unable to read %s: %s"), $filename, $!);
             $asread='';
             while(<IF>) {
                 if (m/^\s*[Tt]his file documents/) {
-                    $asread=$';
+                    $asread = $';
                     last;
                 }
             }
@@ -199,10 +223,10 @@ END
         }
 
         if (!length($description)) {
-            print STDERR "
+            printf STDERR _g("
 No \`START-INFO-DIR-ENTRY' and no \`This file documents'.
-$name: unable to determine description for \`dir' entry - giving up
-";
+%s: unable to determine description for \`dir' entry - giving up
+"), $name;
             exit 1;
         }
 
@@ -250,38 +274,47 @@ $name: unable to determine description for \`dir' entry - giving up
     }
 }
 
-if (!$nowrite && ( ! -e "$infodir/dir" || ! -s "$infodir/dir" )) {
+if (!$nowrite && ( ! -e $dirfile || ! -s _ )) {
     if (-r $backup) {
-	print STDERR "$name: no file $infodir/dir, retrieving backup file $backup.\n";
-	if (system ("cp $backup $infodir/dir")) {
-	    print STDERR "$name: copying $backup to $infodir/dir failed, giving up: $!\n";
+	printf( STDERR _g("%s: no file %s, retrieving backup file %s.")."\n",
+		$name, $dirfile, "$backup" );
+	if (system ('cp', $backup, $dirfile)) {
+	    printf( STDERR _g("%s: copying %s to %s failed, giving up: %s")."\n",
+		    $name, $backup, $dirfile, $! );
 	    exit 1;
 	}
     } else {
         if (-r $default) {
-	    print STDERR "$name: no backup file $backup available, retrieving default file.\n";
-	    
-	    if (system("cp $default $infodir/dir")) {
-		print STDERR "$name: copying $default to $infodir/dir failed, giving up: $!\n";
-	exit 1;
-    }
+	    printf( STDERR _g("%s: no backup file %s available, retrieving default file.")."\n",
+		    $name, $backup );
+
+	    if (system('cp', $default, $dirfile)) {
+		printf( STDERR _g("%s: copying %s to %s failed, giving up: %s")."\n",
+			$name, $default, $dirfile, $! );
+		exit 1;
+	    }
 	} else {
-	    print STDERR "$name: no backup file $backup available.\n";
-	    print STDERR "$name: no default file $default available, giving up.\n";
+	    printf STDERR _g("%s: no backup file %s available.")."\n", $name, $backup;
+	    printf STDERR _g("%s: no default file %s available, giving up.")."\n", $name, $default;
 	    exit 1;
 	}
     }
 }
 
-if (!$nowrite && !link("$infodir/dir","$infodir/dir.lock")) {
-    print STDERR "$name: failed to lock dir for editing! $!\n".
-        ($! =~ m/exists/i ? "try deleting $infodir/dir.lock ?\n" : '');
+if (!$nowrite && !link($dirfile, "$dirfile.lock")) {
+    printf( STDERR _g("%s: failed to lock dir for editing! %s")."\n",
+	    $name, $! );
+    printf( STDERR _g("try deleting %s?")."\n", "$dirfile.lock")
+	if $!{EEXIST};
+    exit 1;
 }
 
-open(OLD,"$infodir/dir") || &ulquit("open $infodir/dir: $!");
+open(OLD, $dirfile) || &ulquit(sprintf(_g("unable to open %s: %s"), $dirfile, $!));
 @work= <OLD>;
-eof(OLD) || &ulquit("read $infodir/dir: $!");
-close(OLD) || &ulquit("close $infodir/dir after read: $!");
+eof(OLD) || &ulquit(sprintf(_g("unable to read %s: %s"), $dirfile, $!));
+close(OLD) || &ulquit(sprintf(_g("unable to close %s after read: %s"),
+				 $dirfile, $!));
+
 while (($#work >= 0) && ($work[$#work] !~ m/\S/)) { $#work--; }
 
 while (@work) {
@@ -312,10 +345,10 @@ if (!$remove) {
 
     if ($i < $j) {
         if ($keepold) {
-            print "$name: existing entry for \`$target_entry' not replaced\n" unless $quiet;
+            printf(_g("%s: existing entry for \`%s' not replaced")."\n", $name, $target_entry) unless $quiet;
             $nowrite=1;
         } else {
-            print "$name: replacing existing dir entry for \`$target_entry'\n" unless $quiet;
+            printf(_g("%s: replacing existing dir entry for \`%s'")."\n", $name, $target_entry) unless $quiet;
         }
         $mss= $i;
         @work= (@work[0..$i-1], @work[$j..$#work]);
@@ -328,10 +361,10 @@ if (!$remove) {
             $mss= $i+1; last;
         }
         if ($mss < 0) {
-            print "$name: creating new section \`$sectiontitle'\n" unless $quiet;
+            printf(_g("%s: creating new section \`%s'")."\n", $name, $sectiontitle) unless $quiet;
             for ($i= $#work; $i>=0 && $work[$i] =~ m/\S/; $i--) { }
             if ($i <= 0) { # We ran off the top, make this section and Misc.
-                print "$name: no sections yet, creating Miscellaneous section too.\n"
+                printf(_g("%s: no sections yet, creating Miscellaneous section too.")."\n", $name)
                     unless $quiet;
                 @work= ("\n", "$sectiontitle\n", "\n", "Miscellaneous:\n", @work);
                 $mss= 1;
@@ -349,7 +382,7 @@ if (!$remove) {
             $mss++;
         }
     } else {
-        print "$name: no section specified for new entry, placing at end\n"
+        printf(_g("%s: no section specified for new entry, placing at end")."\n", $name)
             unless $quiet;
         $mss= $#work+1;
     }
@@ -386,24 +419,27 @@ if (!$remove) {
 
     if ($i < $j) {
         &dprint("i=$i \$work[\$i]='$work[$i]' j=$j \$work[\$j]='$work[$j]'");
-        print "$name: deleting entry \`$match ...'\n" unless $quiet;
+        printf(_g("%s: deleting entry \`%s ...'")."\n", $name, $match) unless $quiet;
         $_= $work[$i-1];
         unless (m/^\s/ || m/^\*/ || m/^$/ ||
                 $j > $#work || $work[$j] !~ m/^\s*$/) {
             s/:?\s+$//;
             if ($keepold) {
-                print "$name: empty section \`$_' not removed\n" unless $quiet;
+                printf(_g("%s: empty section \`%s' not removed")."\n", $name, $_) unless $quiet;
             } else {
                 $i--; $j++;
-                print "$name: deleting empty section \`$_'\n" unless $quiet;
+                printf(_g("%s: deleting empty section \`%s'")."\n", $name, $_) unless $quiet;
             }
         }
         @work= (@work[0..$i-1], @work[$j..$#work]);
     } else {
-        print "$name: no entry for file \`$target_entry'".
-              (length($menuentry) ? " and menu entry \`$menuentry'": '').
-              ".\n"
-            unless $quiet;
+        unless ($quiet) {
+            if (length($menuentry)) {
+                printf _g("%s: no entry for file \`%s' and menu entry \`%s'")."\n", $name, $target_entry, $menuentry;
+            } else {
+                printf _g("%s: no entry for file \`%s'")."\n", $name, $target_entry;
+            }
+        }
     }
 }
 $length = 0;
@@ -443,32 +479,44 @@ foreach ( @work ) {
 }
 
 if (!$nowrite) {
-    open(NEW,"> $infodir/dir.new") || &ulquit("create $infodir/dir.new: $!");
-    print(NEW @head,join("\n",@newwork)) || &ulquit("write $infodir/dir.new: $!");
-    close(NEW) || &ulquit("close $infodir/dir.new: $!");
+    open(NEW,"> $dirfile.new") || &ulquit(sprintf(_g("unable to create %s: %s"),
+						     "$dirfile.new", $!));
+    print(NEW @head,join("\n",@newwork)) ||
+	&ulquit(sprintf(_g("unable to write %s: %s"), "$dirfile.new", $!));
+    close(NEW) || &ulquit(sprintf(_g("unable to close %s: %s"), "$dirfile.new", $!));
 
-    unlink("$infodir/dir.old");
-    link("$infodir/dir","$infodir/dir.old") ||
-        &ulquit("cannot backup old $infodir/dir, giving up: $!");
-    rename("$infodir/dir.new","$infodir/dir") ||
-        &ulquit("install new $infodir/dir: $!");
-unlink("$infodir/dir.lock") || die "$name: unlock $infodir/dir: $!\n";
-system ("cp $infodir/dir $backup") && warn "$name: couldn't backup $infodir/dir in $backup: $!\n";
+    unlink("$dirfile.old");
+    link($dirfile, "$dirfile.old") ||
+	&ulquit(sprintf(_g("unable to backup old %s, giving up: %s"),
+			   $dirfile, $!));
+    rename("$dirfile.new", $dirfile) ||
+	&ulquit(sprintf(_g("unable to install new %s: %s"), $dirfile, $!));
+
+    unlink("$dirfile.lock") ||
+	&quit(sprintf(_g("unable to unlock %s: %s"), $dirfile, $!));
+    system ('cp', $dirfile, $backup) &&
+	warn sprintf(_g("%s: could not backup %s in %s: %s"), $name, $dirfile, $backup, $!)."\n";
+}
+
+sub quit
+{
+    die "$name: $@\n";
 }
 
 sub ulquit {
-    unlink("$infodir/dir.lock") ||
-        warn "$name: warning - unable to unlock $infodir/dir: $!\n";
-    die "$name: $_[0]\n";
+    unlink("$dirfile.lock") ||
+	warn sprintf(_g("%s: warning - unable to unlock %s: %s"),
+		     $name, $dirfile, $!)."\n";
+    &quit($_[0]);
 }
 
 sub checkpipe {
     return if !$pipeit || !$? || $?==0x8D00 || $?==0x0D;
-    die "$name: read $filename: $?\n";
+    &quit(sprintf(_g("unable to read %s: %d"), $filename, $?));
 }
 
 sub dprint {
-    print DEBUG "dbg: $_[0]\n" if ($debug);
+    printf(DEBUG _g("dbg: %s")."\n", $_[0]) if ($debug);
 }
 
 exit 0;
